// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: dpkg-get.cc,v 1.14 1998/03/08 23:21:48 jgg Exp $
/* ######################################################################
   
   dpkg-get - Cover for dpkg
   
   This is an allout cover for dpkg implementing a safer front end. It is
   based largely on pkglib.

   The syntax is different, 
      dpkg-get command args
   Where command is:
      update - Resyncronize the package files from their sources
      upgrade - Smart-Download the newest versions of all packages
      dselectupgrade - Follows dselect's changes to the Status: field
                       and installes new and removes old packages
      distupgrade - Powerfull upgrader designed to handle the issues with
                    a new distribution.
      install - Download and install a given package (by name, not by .deb)
      remove - Remove a given package (by name)
      check - Update the package cache and check for broken packages

   ##################################################################### */
									/*}}}*/
// Include Files							/*{{{*/
#include <iostream.h>
#include <pkglib/error.h>
#include <pkglib/aquire.h>
#include <pkglib/sourcelist.h>
#include <pkglib/aquire.h>
#include <pkglib/depcache.h>
#include <pkglib/packagemanager.h>
#include <pkglib/pkgelement.h>
#include <system.h>

#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <signal.h>
									/*}}}*/

// Aquire Class								/*{{{*/
// ---------------------------------------------------------------------
/* */
class Aquire : public pkgAquire
{
   protected:

   vector<Worker *> Active;
      
   virtual bool RegisterWorker(Worker *Work) 
   {
      Active.push_back(Work);
      return true;
   };
   bool Die() {return false;};
   
   public:
   
   bool Run();
};
									/*}}}*/
// Simulate Class							/*{{{*/
// ---------------------------------------------------------------------
/* This simulate class displays what the ordering code has done and 
   analysis it with a fresh new dependancy cache. In this way we can 
   see all of the effects of an upgrade run */
class Simulate : public pkgPackageManager
{
   protected:
   
   pkgDepCache Sim;
   
   // The Actuall installation implementation
   virtual bool Install(PkgIterator Pkg,string File);
   virtual bool Configure(PkgIterator Pkg);
   virtual bool Remove(PkgIterator Pkg);
   void ShortBreaks();
   
   public:

   Simulate(pkgDepCache &Cache);
};
									/*}}}*/

struct PackageKill
{
   pkgCache::PkgIterator Pkg;
   pkgCache::DepIterator Dep;
};

pkgDepCache *Dep = 0;
bool Check();
bool DoCheck(pkgDepCache &Cache);
bool Install(pkgSourceList &List,int argc,char *argv[]);
bool Update(pkgSourceList &List);
bool Upgrade(pkgSourceList &List);
bool YnPrompt();
void ShowBroken(pkgDepCache &Cache = *Dep);
void DependsDump(pkgCache::VerIterator V);

// Aquire::Run - Run the select loop					/*{{{*/
// ---------------------------------------------------------------------
/* This runs the select loop and monitors the status of all the worker
   threads. */
bool Aquire::Run()
{
   if (pkgAquire::Run() == false)
      return Die();

   struct timeval tv;
   tv.tv_sec = 0;
   tv.tv_usec = 500;
   char S[300];
   S[0] = 0;
   while (Active.empty() == false)
   {
      // Configure the select loop.
      int n = 0;
      fd_set rfds;
      FD_ZERO(&rfds);
      
      for (vector<Worker *>::iterator I = Active.begin(); I != Active.end(); I++)
      {
	 FD_SET((*I)->Fd,&rfds);
	 n = MAX(n,(*I)->Fd);
      }
      
      int Res;
      if ((Res = select(n+1,&rfds,0,0,&tv)) < 0)
      {
	 _error->Errno("select","Error during select");
	 return Die();
      }
      
      /* Run the processes, carefull, Read will change the contents of 
         active and invalidate the iterator */
      for (unsigned int Cur = 0; Cur < Active.size(); Cur++)
      {
	 vector<Worker *>::iterator I = Active.begin() + Cur;
	 
	 if (!FD_ISSET((*I)->Fd,&rfds))
	    continue;

	 if ((*I)->Read() == false)
	 {
	    I = Active.begin() + Cur;
	    delete *I;
	    Active.erase(I);
	    Cur--;
	 }
      }

      if (Res == 0)
      {
	 tv.tv_sec = 0;
	 tv.tv_usec = 500000;
	 
	 cout << S << '\r';
	 S[0] = 0;
	 for (vector<Worker *>::iterator I = Active.begin(); I != Active.end(); I++)
	 {
	    char Tmp[300];
	    char *Mode[] = {"Err","Wgt","Get","Cmp","Prt","Dne"};

	    if ((*I)->Cur != 0)
	    {
	       Item &Cur = *(*I)->Cur;
	       struct stat Buf;
	       if ((*I)->OutputFile.empty() == true ||
		   stat((*I)->OutputFile.c_str(),&Buf) != 0)
		  Buf.st_size = 0;

	       const char *Info = "";
	       if ((*I)->Info.empty() == false)
		  Info = (*I)->Info.c_str();
	       
	       if (Cur.ExpectedSize != 0)
		  sprintf(Tmp,"%s:%s '%s' %lu/%lu ",Mode[Cur.Status],
			  Cur.URN.c_str() + Cur.URN.rfind('/'),Info,
			  Buf.st_size,Cur.ExpectedSize);
	       else
		  sprintf(Tmp,"%s:%s '%s' %lu ",Mode[Cur.Status],
			  Cur.URN.c_str() + Cur.URN.rfind('/'),Info,
			  Buf.st_size);
	    }
	    else
	       sprintf(Tmp,"Forking ");
	    strcat(S,Tmp);
	 }
	 S[78] = 0;
	 cout << S << '\r' << flush;
	 memset(S,' ',strlen(S));
      }
   }
   
   cout << S << '\r';
   bool Ok = true;
   for (vector<Item *>::iterator I = List.begin(); I != List.end(); I++)
   {
      if ((*I)->Status == pkgAquire::Item::Error)
      {
	 cerr << "ERROR " << (*I)->URN << endl;
	 cerr << "  " << (*I)->ErrorText << endl;
	 Ok = false;
      }
   }
   
   return Ok;
};
									/*}}}*/

// Simulate::Simulate - Constructor					/*{{{*/
// ---------------------------------------------------------------------
/* */
Simulate::Simulate(pkgDepCache &Cache) : pkgPackageManager(Cache), 
              Sim(true,true)
{
}
									/*}}}*/
// Simulate::Install - Simulate unpacking of a package			/*{{{*/
// ---------------------------------------------------------------------
/* */
bool Simulate::Install(PkgIterator iPkg,string /*File*/)
{
   // Adapt the iterator
   PkgIterator Pkg = Sim.FindPkg(iPkg.Name());

   cout << "Inst " << Pkg.Name();
   Sim.MarkInstall(Pkg,false);
   
   // Look for broken conflicts+predepends.
   for (PkgIterator I = Sim.PkgBegin(); I.end() == false; I++)
   {
      if (Sim[I].InstallVer == 0)
	 continue;
      
      for (DepIterator D = Sim[I].InstVerIter(Sim).DependsList(); D.end() == false; D++)
	 if (D->Type == pkgDEP_Conflicts || D->Type == pkgDEP_PreDepends)
	    if ((Sim[D] & pkgDepCache::DepInstall) == 0)
	       cout << " [" << I.Name() << " on " << D.TargetPkg().Name() << ']';
   }
   cout << endl;
   return true;
}
									/*}}}*/
// Simulate::Configure - Simulate configuration of a Package		/*{{{*/
// ---------------------------------------------------------------------
/* */
bool Simulate::Configure(PkgIterator iPkg)
{
   // Adapt the iterator
   PkgIterator Pkg = Sim.FindPkg(iPkg.Name());
   
   Sim.MarkInstall(Pkg,false);
   if (Sim[Pkg].InstBroken() == true)
   {
      cout << "Conf " << Pkg.Name() << " broken" << endl;

      ShowBroken(Sim);
      return true;
   }   
   else
      cout << "Conf " << "b" << Sim.BrokenCount() << " i" <<
      Sim.InstCount() << " d" << Sim.DelCount() << ' ' << Pkg.Name();

   if (Sim.BrokenCount() != 0)
      ShortBreaks();
   else
      cout << endl;
   
   return true;
}
									/*}}}*/
// Simulate::Remove - Simulate the removal of a package			/*{{{*/
// ---------------------------------------------------------------------
/* */
bool Simulate::Remove(PkgIterator iPkg)
{
   // Adapt the iterator
   PkgIterator Pkg = Sim.FindPkg(iPkg.Name());

   Sim.MarkDelete(Pkg);
   cout << "Remv " << "b" << Sim.BrokenCount() << " i" <<
      Sim.InstCount() << " d" << Sim.DelCount() << ' ' << Pkg.Name();

   if (Sim.BrokenCount() != 0)
      ShortBreaks();
   else
      cout << endl;

   return true;
}
									/*}}}*/
// Simulate::ShortBreaks - Print out a short line describing all breaks	/*{{{*/
// ---------------------------------------------------------------------
/* */
void Simulate::ShortBreaks()
{
   cout << " [";
   for (PkgIterator I = Sim.PkgBegin(); I.end() == false; I++)
   {
      if (Sim[I].InstBroken() == true)
	 cout << I.Name() << ' ';
   }
   cout << ']' << endl;
}
									/*}}}*/

// YnPrompt - Yes No Prompt.						/*{{{*/
// ---------------------------------------------------------------------
/* Returns true on a Yes.*/
bool YnPrompt()
{
   char C = 0;
   read(STDIN_FILENO,&C,1);
   if (!(C == 'Y' || C == 'y' || C == '\n' || C == '\r'))
      return false;
   return true;
}
									/*}}}*/
// Stats - Show some statistics						/*{{{*/
// ---------------------------------------------------------------------
/* */
void Stats()
{
   unsigned long Upgrade = 0;
   unsigned long Install = 0;
   for (pkgCache::PkgIterator I = Dep->PkgBegin(); I.end() == false; I++)
   {
      if ((*Dep)[I].NewInstall() == true)
	 Install++;
      else
	 if ((*Dep)[I].Upgrade() == true)
	    Upgrade++;
   }   
   
   cout << Upgrade << " packages upgraded, " << 
      Install << " newly installed packages, " << 
      Dep->DelCount() << " packages to remove." << endl;
   cout << Dep->DebSize() << " bytes to download, " << 
      Dep->UsrSize() << " bytes needed after install." << endl;
}
									/*}}}*/
// ShowBroken - Debugging aide						/*{{{*/
// ---------------------------------------------------------------------
/* */
void ShowBroken(pkgDepCache &Cache)
{
   cout << "Sorry, but doing what you asked would break the following packages:" << endl;
   pkgCache::PkgIterator I = Cache.PkgBegin();
   for (;I.end() != true; I++)
   {
      if (Cache[I].InstBroken() == true)
      {
	 // Print out each package and the failed dependancies
	 cout <<"  " <<  I.Name() << ":";
	 for (pkgCache::DepIterator D = Cache[I].InstVerIter(Cache).DependsList(); D.end() == false; D++)
	 {
	    if (Dep->IsImportantDep(D) == false || (Cache[D] &
						    pkgDepCache::DepInstall) != 0)
	       continue;
	    
	    if (D->Type == pkgDEP_Conflicts)
	       cout << " Conflicts:" << D.TargetPkg().Name();
	    else
	       cout << " Depends:" << D.TargetPkg().Name();
	 }	    
	 cout << endl;
      }   
   }   
}
									/*}}}*/
// ShowNew - Show packages to newly install				/*{{{*/
// ---------------------------------------------------------------------
/* */
void ShowNew()
{
   /* Print out a list of packages that are going to be removed extra
      to what the user asked */
   if (Dep->DelCount() != 0)
   {
      cout << "The following NEW packages will installed:" << endl;
      cout << "  ";
      pkgCache::PkgIterator I = Dep->PkgBegin();
      int Count = 0;
      for (;I.end() != true; I++)
	 if ((*Dep)[I].NewInstall() == true)
         {
	    cout << I.Name() << ' ';
	    Count++;
	    if (Count % 7 == 0)
	       cout << endl << "  ";
	 }
      
      cout << endl;
   }
}
									/*}}}*/
// ShowDel - Show packages to delete					/*{{{*/
// ---------------------------------------------------------------------
/* */
void ShowDel()
{
   /* Print out a list of packages that are going to be removed extra
      to what the user asked */
   if (Dep->DelCount() != 0)
   {
      cout << "The following packages will be REMOVED:" << endl;
      cout << "  ";
      pkgCache::PkgIterator I = Dep->PkgBegin();
      int Count = 0;
      for (;I.end() != true; I++)
	 if ((*Dep)[I].Delete() == true)
         {
	    cout << I.Name() << ' ';
	    Count++;
	    if (Count % 7 == 0)
	       cout << endl << "  ";
	 }
      cout << endl;
   }
}
									/*}}}*/
// DoArchives - Fetch all of the required archives and do the install	/*{{{*/
// ---------------------------------------------------------------------
/* */
bool DoArchives(pkgSourceList &List)
{
//   pkgPackageManager PM(*Dep);
   Simulate PM(*Dep);
   return PM.DoInstall();
   
/*   Aquire Get;
   
   // Generate the archive list
   if (PM.GetArchives(List,Get) == false)
      return false;
   
   // Download the archives
   if (Get.Run() == false)
      return false;*/
   
//   return true;
}
									/*}}}*/

// Update - Download newer package files				/*{{{*/
// ---------------------------------------------------------------------
/* */
bool Update(pkgSourceList &List)
{
   Aquire Get;
   if (pkgUpdateMeta(List,Get) == false)
      return false;

   if (Get.Clean() == false)
      return false;
   
   if (Get.Run() == false)
      return false;

   cout << "Updating package file cache... " << flush;
   if (pkgMakeSrcCache(List) == false)
   {
      cout << endl;
      return false;
   }

   cout << "Done" << endl;

   return Check();
}
									/*}}}*/
// Check - Check the system for problems				/*{{{*/
// ---------------------------------------------------------------------
/* */
bool Check()
{
   if (Dep != 0)
      return true;
   
   cout << "Updating package status cache... " << flush;
   if (pkgMakeStatusCache() == false)
   {
      cout << "Error" << endl;
      _error->Error("You should probably re-run dpkg-get update");
      return false;
   }
   
   cout << "Done" << endl;
      
   cout << "Checking system integrity... " << flush;

   Dep = new pkgDepCache(true,true);
   if (_error->PendingError() == true)
   {
      delete Dep;
      Dep = 0;
      cout << "Bad Cache" << endl;
      return false;
   }
   
   if (DoCheck(*Dep) == false)      
   {
      cout << "Error"<< endl;
      return false;
   }
   
   cout << "Ok" << endl;
   
   return true;
}
									/*}}}*/
// DoCheck - Check the system for problems				/*{{{*/
// ---------------------------------------------------------------------
/* */
bool DoCheck(pkgDepCache &Cache)
{   
   if (Cache.DelCount() != 0 || Cache.InstCount() != 0)
      return _error->Error("Internal Error, zero counts");
   
   // Nothing is broken
   if (Cache.BrokenCount() == 0)
      return true;

   _error->Error("%i broken packages",Cache.BrokenCount());
   pkgCache::PkgIterator I = Cache.PkgBegin();
   for (;I.end() != true; I++)
      if (Cache[I].NowBroken() == true)
	 _error->Error("Package %s is in a bad state",I.Name());
   return false;
}
									/*}}}*/
// Install - Install Packages						/*{{{*/
// ---------------------------------------------------------------------
/* */
bool Install(pkgSourceList &List,int argc,char *argv[])
{
   // Setup Dep
   if (Check() == false || Dep == 0)
      return false;
   
   int ExpectedInst = 0;
   
   for (int I = 0; I != argc; I++)
   {
      // Locate the package
      pkgCache::PkgIterator Pkg = Dep->FindPkg(argv[I]);
      if (Pkg.end() == true)
	 return _error->Error("Couldn't find package %s",argv[I]);
      
      // Check if there is something new to install
      pkgDepCache::StateCache &State = (*Dep)[Pkg];
      if (State.CandidateVer == 0)
	 return _error->Error("Package %s has no installation candidate",argv[I]);
      
      // Install it
      Dep->MarkInstall(Pkg,false);
      if (State.Install() == false)
	 cout << "Sorry, " << argv[I] << " is allready the newest version"  << endl;
      else
	 ExpectedInst++;

      // Hmm...
      if (State.InstBroken() == true)
      {
	 cout << "Package " << argv[I] << " has unmet dependencies." << endl;
	 cout << " Do you want to try auto installing ? [Y/n]" << flush;

	 if (YnPrompt() == false)
	    return false;
	 
	 // Install it with autoinstalling enabled.
	 Dep->MarkKeep(Pkg);
	 Dep->MarkInstall(Pkg,true);
      }
   }

   /* Print out a list of packages that are going to be installed extra
      to what the user asked */
   if (Dep->InstCount() != ExpectedInst)
   {
      cout << "The following extra packages will be installed:" << endl;
      cout << "  ";
      pkgCache::PkgIterator I = Dep->PkgBegin();
      for (;I.end() != true; I++)
      {
	 if ((*Dep)[I].Install() == true)
         {
	    // See if the package was requested to be installed
	    int J = 0;
	    for (; J != argc; J++)
	       if (strcmp(argv[J],I.Name()) == 0)
		  break;
	    if (J == argc)
	       cout << I.Name() << ' ';
	 }
      }      
      cout << endl;
   }   

   ShowDel();
   
   // Now we check the state of the packages,
   if (Dep->BrokenCount() != 0)
   {
      ShowBroken();
      return false;
   }   

   Stats();
   
   // Check, just to be sure.
   if (Dep->InstCount() != ExpectedInst || Dep->DelCount() != 0)
   {
      cout << "Do you want to continue? [Y/n]" << flush;
      if (YnPrompt() == false)
	 return false;
   }

   return DoArchives(List);
}
									/*}}}*/
// Upgrade - Update to the newest packages				/*{{{*/
// ---------------------------------------------------------------------
/* */
bool Upgrade(pkgSourceList &List)
{
   // Setup Dep
   if (Check() == false || Dep == 0)
      return false;

   bool PreBroke = Dep->BrokenCount() != 0;
   
   // Do the upgrade
   Dep->AllUpgrade();
   
   // Safety check
   if (Dep->BrokenCount() != 0)
   {
      ShowBroken();
      if (PreBroke == true)
	 return true;
	  
      return _error->Error("Internal Error, AllUpgrade broke stuff");
   }
   
   // Generate the report
   pkgCache::PkgIterator I = Dep->PkgBegin();
   bool First = true;
   for (;I.end() != true; I++)
   {
      // Not interesting
      if ((*Dep)[I].Upgrade() == true || (*Dep)[I].Upgradable() == false ||
	  I->CurrentVer == 0)
	 continue;
      if (First == true)
      {
	 cout << "The following packages have been kept back because of conflicts" << endl;
	 cout << "   ";
	 First = false;
      }
      
      cout << I.Name() << " ";
   }
   if (First == false)
      cout << endl;
   
   Stats();

   cout << "Do you want to continue? [Y/n]" << flush;
   if (YnPrompt() == false)
      return false;
   
   return DoArchives(List);
}
									/*}}}*/
// DselectUpgrade - Upgrade using dselect new install information	/*{{{*/
// ---------------------------------------------------------------------
/* */
bool DselectUpgrade(pkgSourceList &List)
{
   // Setup Dep
   if (Check() == false || Dep == 0)
      return false;

   // Install everything with the install flag set
   pkgCache::PkgIterator I = Dep->PkgBegin();
   for (;I.end() != true; I++)
   {
      if (I->CurrentVer == 0)
	 continue;
      
      /* Install the package only if it is a new install, the autoupgrader
         will deal with the rest */
      if (I->SelectedState == pkgSTATE_Install)
	 Dep->MarkInstall(I);

      // Remove packages 
      if (I->SelectedState == pkgSTATE_DeInstall ||
	  I->SelectedState == pkgSTATE_Purge)
	 Dep->MarkDelete(I);
   }

   /* Use updates smart upgrade to do the rest, it will automatically
      ignore held items */
   return Upgrade(List);
}
									/*}}}*/
// DistUpgrade - Smart upgrade to a new distribution			/*{{{*/
// ---------------------------------------------------------------------
/* */
signed short *SortScores = 0;
int ScoreSort(const void *a,const void *b)
{
   pkgCache::Package const **A = (pkgCache::Package const **)a;
   pkgCache::Package const **B = (pkgCache::Package const **)b;
   if (SortScores[(*A)->ID] > SortScores[(*B)->ID])
      return -1;
   if (SortScores[(*A)->ID] < SortScores[(*B)->ID])
      return 1;
   return 0;
}
bool DistUpgrade(pkgSourceList &List)
{
   // Setup Dep
   if (Check() == false || Dep == 0)
      return false;
   
   pkgDepCache &Cache = *Dep;
   
   /* Auto upgrade all installed packages, this provides the basis 
      for the installation */
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
      if (I->CurrentVer != 0)
	 Cache.MarkInstall(I,true);
   
   /* Now, auto upgrade all essential packages - this ensures that
      the essential packages are present and working */
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
      if ((I->Flags & pkgFLAG_Essential) == pkgFLAG_Essential)
	 Cache.MarkInstall(I,true);
   
   /* We do it again to get all essential packages installed, even if it
      causes inter-essential conflicts - those will be resolved below */
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
      if ((I->Flags & pkgFLAG_Essential) == pkgFLAG_Essential)
	 Cache.MarkInstall(I,false);

   /* Now generate a set of scores for each package, this is how we
      determine what is best to remove. For now we are using a static
      scoring system, but a dynamically adjusting one might work better..
    */
   unsigned long Size = Cache.HeaderP->PackageCount;
   signed short *Scores = new signed short[Size];
   memset(Scores,0,sizeof(*Scores)*Size);
   
   // Generate the base scores for a package based on its properties
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
   {
      if (Cache[I].InstallVer == 0)
	 continue;
      
      signed short &Score = Scores[I->ID];
      
      /* This is arbitary, it should be high enough to elevate an
         essantial package above most other packages but low enough
	 to allow an obsolete essential packages to be removed by
	 a conflicts on a powerfull normal package (ie libc6) */
      if ((I->Flags & pkgFLAG_Essential) == pkgFLAG_Essential)
	 Score += 100;

      // We transform the priority
      // Important Required Standard Optional Extra
      signed short PrioMap[] = {0,3,2,1,-1,-2};
      if (Cache[I].InstVerIter(Cache)->Priority <= 5)
	 Score += PrioMap[Cache[I].InstVerIter(Cache)->Priority];
   }

   // Now that we have the base scores we go and propogate dependancies
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
   {
      if (Cache[I].InstallVer == 0)
	 continue;
      
      for (pkgCache::DepIterator D = Cache[I].InstVerIter(Cache).DependsList(); D.end() == false; D++)
      {
	 if (D->Type == pkgDEP_Depends || D->Type == pkgDEP_PreDepends)
	    Scores[D.TargetPkg()->ID]++;
      }
   }   
   
   // Copy the scores to advoid additive looping
   signed short *OldScores = new signed short[Size];
   memcpy(OldScores,Scores,sizeof(*Scores)*Size);
      
   /* Now we cause 1 level of dependancy inheritance, that is we add the 
      score of the packages that depend on the target Package. This 
      fortifies high scoring packages */
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
   {
      if (Cache[I].InstallVer == 0)
	 continue;
      
      for (pkgCache::DepIterator D = I.RevDependsList(); D.end() == false; D++)
      {
	 // Only do it for the install version
	 if ((pkgCache::Version *)D.ParentVer() != Cache[D.ParentPkg()].InstallVer ||
	     (D->Type != pkgDEP_Depends && D->Type != pkgDEP_PreDepends))
	    continue;	 
	 
	 Scores[I->ID] += abs(OldScores[D.ParentPkg()->ID]);
      }      
   }

   /* Now we propogate along provides. This makes the packages that 
      provide important packages extremely important */
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
   {
      for (pkgCache::PrvIterator P = I.ProvidesList(); P.end() == false; P++)
      {
	 // Only do it once per package
	 if ((pkgCache::Version *)P.OwnerVer() != Cache[P.OwnerPkg()].InstallVer)
	    continue;
	 Scores[P.OwnerPkg()->ID] += abs(Scores[I->ID] - OldScores[I->ID]);
      }      
   }
   delete [] OldScores;   
   
   /* We have to order the packages so that the broken fixing pass 
      operates from highest score to lowest. This prevents problems when
      high score packages cause the removal of lower score packages that
      would cause the removal of even lower score packages. */
   pkgCache::Package **PList = new pkgCache::Package *[Size];
   pkgCache::Package **PEnd = PList;
   for (pkgCache::PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
      *PEnd++ = I;
   SortScores = Scores;
   qsort(PList,PEnd - PList,sizeof(*PList),&ScoreSort);
   
/* for (pkgCache::Package **K = PList; K != PEnd; K++)
      if (Scores[(*K)->ID] != 0)
	 cout << Scores[(*K)->ID] << ' ' << pkgCache::PkgIterator(Cache,*K).Name() << endl;*/
      
   /* Now consider all broken packages. For each broken package we either
      remove the package or fix it's problem. We do this once, it should
      not be possible for a loop to form (that is a < b < c and fixing b by
      changing a breaks c) */
   bool Change = true;
   for (int Counter = 0; Counter != 10 && Change == true; Counter++)
   {
      Change = false;
      for (pkgCache::Package **K = PList; K != PEnd; K++)
      {
	 pkgCache::PkgIterator I(Cache,*K);

	 if (Cache[I].InstallVer == 0 || Cache[I].InstBroken() == false)
	    continue;
	 
	 // Isolate the problem dependancy
	 PackageKill KillList[100];
	 PackageKill *LEnd = KillList;
	 for (pkgCache::DepIterator D = Cache[I].InstVerIter(Cache).DependsList(); D.end() == false;)
	 {
	    // Compute a single dependancy element (glob or)
	    pkgCache::DepIterator Start = D;
	    pkgCache::DepIterator End = D;
	    unsigned char State = 0;
	    for (bool LastOR = true; D.end() == false && LastOR == true; D++)
	    {
	       State |= Cache[D];
	       LastOR = (D->CompareOp & pkgOP_OR) == pkgOP_OR;
	       if (LastOR == true)
		  End = D;
	    }
	    
	    // We only worry about critical deps.
	    if (End.IsCritical() != true)
	       continue;
	    
	    // Dep is ok
	    if ((Cache[End] & pkgDepCache::DepGInstall) == pkgDepCache::DepGInstall)
	       continue;
	    
	    // Hm, the group is broken.. I have no idea how to handle this
	    if (Start != End)
	    {
	       Cache.MarkDelete(I);
	       break;
	    }
	    	    
	    /* Conflicts is simple, decide if we should remove this package
	     or the conflicted one */
	    // Must consider provides!
	    if (Scores[I->ID] <= Scores[End.TargetPkg()->ID] ||
		((Cache[End] & pkgDepCache::DepGNow) == 0 &&
		 End->Type != pkgDEP_Conflicts))
	    {
	       // See if a keep will do
	       Cache.MarkKeep(I);
	       if (Cache[I].InstBroken() == false)
		  cout << "Holding Back " << I.Name() << " In favor of " << End.TargetPkg().Name() << endl;
	       else
	       {
		  cout << "Suiciding " << I.Name() << " In favor of " << End.TargetPkg().Name() << endl;
		  Cache.MarkDelete(I);
	       }
	       
	       Change = true;
	       break;
	    }
	    else
	    {
	       LEnd->Pkg = End.TargetPkg();
	       LEnd->Dep = End;
	       LEnd++;
	    }
	 }
	 
	 // Apply the kill list now
	 if (Cache[I].InstallVer != 0)
	    for (PackageKill *J = KillList; J != LEnd; J++)
         {
	    cout << "Killing " << J->Pkg.Name() << " In favor of " << I.Name() << endl;
	    Change = true;
	    if ((Cache[J->Dep] & pkgDepCache::DepGNow) == 0)
	    {
	       if (J->Dep->Type == pkgDEP_Conflicts)
		  Cache.MarkDelete(J->Pkg);
	    }
	    else
	       Cache.MarkKeep(J->Pkg);
	 }      
      }
   }

   if (Dep->BrokenCount() != 0)
   {
      ShowBroken();
      return _error->Error("Internal error, upgrader generated breaks.");
   }
   
   ShowDel();
   ShowNew();
   cout << "Kept " << Cache.KeepCount() << endl;
   Stats();
   
   delete [] Scores;
   delete [] PList;
   return DoArchives(List);
}
									/*}}}*/

int main(int argc,char *argv[])
{
   if (argc <= 1)
   {
      cerr << "You must specify a command: update, install, check or remove" << endl;
      return 100;
   }
   
   // We will need the sourcelist
   pkgSourceList List;
   int Res = -1;
   while (1)
   {
      if (List.ReadMainList() == false)
      {
	 Res = 0;
	 cerr << "Unable to read the main source list" << endl;
	 break;
      }
   
      if (strcmp(argv[1],"update") == 0)
	 Res = Update(List);
      
      if (strcmp(argv[1],"check") == 0)
	 Res = Check();

      if (strcmp(argv[1],"install") == 0)
	 Res = Install(List,argc-2,argv+2);
      
      if (strcmp(argv[1],"upgrade") == 0)
	 Res = Upgrade(List);

      if (strcmp(argv[1],"dselect-upgrade") == 0)
	 Res = DselectUpgrade(List);

      if (strcmp(argv[1],"dist-upgrade") == 0)
	 Res = DistUpgrade(List);
      
      break;
   }

   if (Res == -1)
   {
      cerr << "Unknown command " << argv[1] << endl;
      return 100;
   }
   
   // An error
   if (Res == 0)
   {
      _error->DumpErrors();
      return 100;
   }     

   return 0;   
}
