// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: packagemanager.cc,v 1.12 1998/03/10 07:57:47 jgg Exp $
/* ######################################################################

   Package Manager - Abstacts the package manager

   More work is needed in the area of transitioning provides, ie exim
   replacing smail. This can cause interesing side effects.

   Other cases involving conflicts+replaces should be tested. 
   
   ##################################################################### */
									/*}}}*/
// Include Files							/*{{{*/
#include <pkglib/packagemanager.h>
#include <pkglib/depcache.h>
#include <pkglib/sourcelist.h>
#include <pkglib/aquire.h>
#include <pkglib/controlcache.h>
#include <pkglib/error.h>
#include <options.h>
									/*}}}*/

pkgPackageManager *pkgPackageManager::Me = 0;

// Smart pointers							/*{{{*/
struct ColorList
{
   typedef unsigned char sbool;
   sbool *List;
   
   operator sbool *() {return List;};
   operator void *() {return List;};
   sbool &operator [](int I) {return List[I];};
   void Die() {delete List; List = 0;};
   
   ColorList(pkgCache &Cache) {List = new sbool[Cache.HeaderP->PackageCount];};
   ~ColorList() {delete List;};
};

struct PkgList
{
   pkgCache::Package **List;
   
   operator pkgCache::Package **() {return List;};
   pkgCache::Package *&operator [](int I) {return List[I];};
   void Die() {delete List; List = 0;};
   
   PkgList(pkgCache &Cache) {List = new pkgCache::Package *[Cache.HeaderP->PackageCount];};
   ~PkgList() {delete List;};
};
									/*}}}*/

// PM::PackageManager - Constructor					/*{{{*/
// ---------------------------------------------------------------------
/* */
pkgPackageManager::pkgPackageManager(pkgDepCache &Cache) : Cache(Cache)
{
   FileNames = new string[Cache.Head().PackageCount];
}
									/*}}}*/
// PM::PackageManager - Destructor					/*{{{*/
// ---------------------------------------------------------------------
/* */
pkgPackageManager::~pkgPackageManager()
{
   delete [] FileNames;
}
									/*}}}*/
// PM::GetArchives - Puts the install archives in the aquire list	/*{{{*/
// ---------------------------------------------------------------------
/* The Source list is used to convert the packages to install into
   URNs which are then passed to Aquire to convert to FileNames. */
bool pkgPackageManager::GetArchives(pkgSourceList &List,pkgAquire &Engine)
{
   pkgControlCache CCache(Cache);
   if (_error->PendingError() == true)
      return false;

   Engine.OutputDir(PKG_DEB_CA_ARCHIVES);
   pkgCache::PkgIterator I = Cache.PkgBegin();
   for (;I.end() != true; I++)
   {
      // Not interesting
      if (Cache[I].InstallVer == (pkgCache::Version *)I.CurrentVer() || Cache[I].Delete() == true)
	 continue;
      
      // Cross match with the source list.
      pkgSourceList::const_iterator Dist = List.MatchPkgFile(Cache[I].InstVerIter(Cache));
      if (Dist == List.end())
      {
	 _error->Warning("Couldn't locate an archive source for package %s",I.Name());
	 continue;
      }

      // Read in the info record
      pkgSPkgCtrlInfo Inf = CCache[Cache[I].InstVerIter(Cache)];
      if (Inf.isNull() == true)
      {
	 _error->Warning("Couldn't locate info for package %s",I.Name());
	 continue;
      }    

      // Isolate the filename
      string File = Inf->Find("Filename")->Value();
      if (File.empty() == true)
      {
	 _error->Warning("Couldn't locate an archive for package %s",I.Name());
	 continue;
      }    

      // Generate the get request.
      string URN = Dist->ArchiveURN(File);
      
      // Output file
      unsigned int Pos = File.rfind('/');
      if (Pos == File.length())
	 return _error->Error("Malformed file line in package %s",I.Name());
      
      // Null pos isnt used in present package files
      if (Pos == string::npos)
	 Pos = 0;
      else
	 Pos++;
      
      if (Engine.Get(URN,string(File,Pos),
		     Cache[I].InstVerIter(Cache)->Size,
		     Inf->Find("MD5sum")->Value(),
		     &FileNames[I->ID]) == false)
	    return false;
   }

   return true;
}
									/*}}}*/

// PM::VisitDeps - Visit forward install dependancies			/*{{{*/
// ---------------------------------------------------------------------
/* */
bool pkgPackageManager::VisitDeps(VisitState &State,DepFunc F,
				  PkgIterator Pkg)
{
   if (F == 0 || Pkg.end() == true || Cache[Pkg].InstallVer == 0)
      return true;
   
   return (this->*F)(State,Cache[Pkg].InstVerIter(Cache).DependsList());
}
									/*}}}*/
// PM::VisitRDeps - Visit reverse dependancies				/*{{{*/
// ---------------------------------------------------------------------
/* */
bool pkgPackageManager::VisitRDeps(VisitState &State,DepFunc F,
				   PkgIterator Pkg)
{
   if (F == 0 || Pkg.end() == true)
      return true;
   
   return (this->*F)(State,Pkg.RevDependsList());
}
									/*}}}*/
// PM::VisitRProvides - Visit provides reverse dependancies		/*{{{*/
// ---------------------------------------------------------------------
/* */
bool pkgPackageManager::VisitRProvides(VisitState &State,DepFunc F,
				       VerIterator Ver)
{
   if (F == 0 || Ver.end() == true)
      return true;
   
   bool Res = true;
   for (PrvIterator P = Ver.ProvidesList(); P.end() == false; P++)
      Res &= (this->*F)(State,P.ParentPkg().RevDependsList());
   return true;
}
									/*}}}*/
// PM::VisitProvides - Visit all of the providing packages		/*{{{*/
// ---------------------------------------------------------------------
/* This routine calls visit on all providing packages. */
bool pkgPackageManager::VisitProvides(VisitState &State,DepIterator D)
{
   PkgIterator Pkg = D.TargetPkg();
   PrvIterator Prv = Pkg.ProvidesList();   
   while (1)
   {	       
      if (VisitNode(State,Pkg) == false)
	 return false;
      
      // Check versioned provides
      for (; Prv.end() == false; Prv++)
	 if (pkgCheckDep(D.TargetVer(),Prv.ProvideVersion(),D->CompareOp) == true)
	    break;
      
      // No more
      if (Prv.end() == true)
	 break;
      
      Pkg = Prv.OwnerPkg();
      Prv++;
   }
   return true;
}
									/*}}}*/
// PM::VisitNode - Recursive ordering director				/*{{{*/
// ---------------------------------------------------------------------
/* This is the core ordering routine. It calls the set dependancy
   consideration functions which then potentiall call this again. Finite
   depth is achived through the colouring mechinism. */
static int Depth = 0;
bool pkgPackageManager::VisitNode(VisitState &State,PkgIterator Pkg)
{
   // Looping or irrelevent.
   if (Pkg.end() == true || State.Colors[Pkg->ID] != 0 || 
       Cache[Pkg].Keep() == true)
      return true;

/* for (int j = 0; j != Depth; j++) cout << ' ';
 cout << "Visit " << Pkg.Name() << endl;
   Depth++;*/
   
   // Color grey
   State.Colors[Pkg->ID] = 2;

   VisitState Old = State;
   bool Res = true;
   if (Cache[Pkg].Delete() == false)
   {
      // Primary
      Res &= Res && VisitDeps(State,State.Primary,Pkg);
      Res &= Res && VisitRDeps(State,State.Primary,Pkg);
      Res &= Res && VisitRProvides(State,State.Primary,Pkg.CurrentVer());
      Res &= Res && VisitRProvides(State,State.Primary,Cache[Pkg].InstVerIter(Cache));
      
      // RevDep
      Res &= Res && VisitRDeps(State,State.RevDepends,Pkg);
      Res &= Res && VisitRProvides(State,State.RevDepends,Pkg.CurrentVer());
      Res &= Res && VisitRProvides(State,State.RevDepends,Cache[Pkg].InstVerIter(Cache));
	
      // Secondary
      Res &= Res && VisitDeps(State,State.Secondary,Pkg);
      Res &= Res && VisitRDeps(State,State.Secondary,Pkg);
      Res &= Res && VisitRProvides(State,State.Secondary,Pkg.CurrentVer());
      Res &= Res && VisitRProvides(State,State.Secondary,Cache[Pkg].InstVerIter(Cache));
   }
   else
   { 
      // RevDep
      Res &= Res && VisitRDeps(State,State.Remove,Pkg);
      Res &= Res && VisitRProvides(State,State.Remove,Pkg.CurrentVer());
   }

   if (State.Colors[Pkg->ID] != 1)
   {
      State.Colors[Pkg->ID] = 1;
      *State.Order = Pkg;
      State.Order++;
   }
   
/* Depth--;
   for (int j = 0; j != Depth; j++) cout << ' ';
   cout << "Leave " << Pkg.Name() << ' ' << (int)State.Colors[Pkg->ID] << endl;*/

//   VisitDeps(State,State.RevDepends,Pkg);
   
   return true;
}
									/*}}}*/

// PM::DepUnPackCrit - Critical UnPacking ordering			/*{{{*/
// ---------------------------------------------------------------------
/* Critical unpacking ordering strives to satisfy Conflicts: and 
   PreDepends: only. When a prdepends is encountered the Primary 
   DepFunc is changed to be DepUnPackPreD. 

   This is largely a duplication of DepUnPackPreD
 
   Loops are preprocessed and logged. */
bool pkgPackageManager::DepUnPackCrit(VisitState &State,DepIterator D)
{
   for (; D.end() == false; D++)
   {
      if (D.Reverse() == true)
      {
	 /* Reverse depenanices are only interested in conflicts, 
	    predepend breakage is ignored here */
	 if (D->Type != pkgDEP_Conflicts)
	    continue;

	 // Duplication elimination, consider only the current version
	 if (D.ParentPkg().CurrentVer() != D.ParentVer())
	    continue;
	 
	 /* For reverse dependancies we wish to check if the
	    dependancy is satisifed in the install state. The
	    target package (caller) is going to be in the installed
	    state. */
	 if ((Cache[D] & pkgDepCache::DepInstall) != 0)
	    continue;

	 if (VisitNode(State,D.ParentPkg()) == false)
	    return false;
      }
      else
      {
	 /* Forward critical dependancies MUST be correct before the 
	    package can be unpacked. */
	 if (D->Type != pkgDEP_Conflicts && D->Type != pkgDEP_PreDepends)
	    continue;
	 	 	 
	 /* We wish to check if the dep is okay in the now state of the
	    target package against the install state of this package. */
	 if ((Cache[D] & pkgDepCache::DepNow) != 0)
	 {
	    /* We want to catch predepends loops with the code below.
	       Conflicts loops that are Dep OK are ignored */
	    if (State.Colors[D.TargetPkg()->ID] != 2 ||
		D->Type != pkgDEP_PreDepends)
	       continue;
	 }

	 // This is the loop detection
	 if (State.Colors[D.TargetPkg()->ID] != 0)
	 {
	    if (State.Colors[D.TargetPkg()->ID] == 2)
	       cout << "crit looping " << D.TargetPkg().Name() << " for " << D.ParentPkg().Name() << endl;
	    continue;
	 }

	 /* Predepends require a special ordering stage, they must have
	    all dependants installed as well */
	 DepFunc Old = State.Primary;
	 bool Res = false;
	 if (D->Type == pkgDEP_PreDepends)
	    State.Primary = &DepUnPackPreD;
	 Res = VisitProvides(State,D);
	 State.Primary = Old;
	 if (Res == false)
	    return false;
      }	 
   }   
   return true;
}
									/*}}}*/
// PM::DepUnPackPreD - Critical UnPacking ordering with depends		/*{{{*/
// ---------------------------------------------------------------------
/* Critical PreDepends (also configure immediate and essential) strives to
   ensure not only that all conflicts+predepends are met but that this
   package will be immediately configurable when it is unpacked. 

   This is largely a duplication of DepUnPackCrit
   
   Loops are preprocessed and logged. */
bool pkgPackageManager::DepUnPackPreD(VisitState &State,DepIterator D)
{
   for (; D.end() == false; D++)
   {
      if (D.Reverse() == true)
      {
	 /* Reverse depenanices are only interested in conflicts, 
	    predepend breakage is ignored here */
	 if (D->Type != pkgDEP_Conflicts)
	    continue;
	 
	 // Duplication elimination, consider the current version
	 if (D.ParentPkg().CurrentVer() != D.ParentVer())
	    continue;
	 
	 /* For reverse dependancies we wish to check if the
	    dependancy is satisifed in the install state. The
	    target package (caller) is going to be in the installed
	    state. */
	 if ((Cache[D] & pkgDepCache::DepInstall) != 0)
	    continue;

	 if (VisitNode(State,D.ParentPkg()) == false)
	    return false;
      }
      else
      {
	 if (D.IsCritical() == false)
	    continue;
	 	 	 
	 /* We wish to check if the dep is okay in the now state of the
	    target package against the install state of this package. */
	 if ((Cache[D] & pkgDepCache::DepNow) != 0)
	 {
	    /* We want to catch predepends loops with the code below.
	       Conflicts loops that are Dep OK are ignored */
	    if (State.Colors[D.TargetPkg()->ID] != 2 ||
		D->Type != pkgDEP_PreDepends)
	       continue;
	 }
	 
	 /* We dont even attempt to consider packages already under
	    consideration. */
	 if (State.Colors[D.TargetPkg()->ID] != 0)
	 {
	    if (State.Colors[D.TargetPkg()->ID] == 2)
	       cout << "pred looping " << D.TargetPkg().Name() << " for " << D.ParentPkg().Name() << endl;
	    continue;
	 }

	 if (VisitProvides(State,D) == false)
	    return false;
      }
   }   
   return true;
}
									/*}}}*/
// PM::DepUnPackDep - Reverse dependancy considerations			/*{{{*/
// ---------------------------------------------------------------------
/* Reverse dependancies are considered to determine if unpacking this
   package will break any existing dependancies. If so then those
   packages are ordered before this one so that they are in the
   UnPacked state. 
 
   The forwards depends loop is designed to bring the packages dependants
   close to the package. This helps reduce deconfigure time. 
   
   Loops are irrelevent to this. */
bool pkgPackageManager::DepUnPackDep(VisitState &State,DepIterator D)
{
   for (; D.end() == false; D++)
      if (D.IsCritical() == true)
      {
	 if (D.Reverse() == true)
	 {
	    /* Duplication prevention. We consider rev deps only on
	       the current version, a not installed package
	       cannot break */
	    if (D.ParentPkg()->CurrentVer == 0 ||
		D.ParentPkg().CurrentVer() != D.ParentVer())
	       continue;

	    // The dep will not break so it is irrelevent.
	    if ((Cache[D] & pkgDepCache::DepInstall) != 0)
	       continue;
	    
	    if (VisitNode(State,D.ParentPkg()) == false)
	       return false;
	 }
	 else
	    if (D->Type == pkgDEP_Depends)
	       if (VisitProvides(State,D) == false)
		  return false;
      }
   return true;
}
									/*}}}*/
// PM::DepConfigure - Configuration ordering				/*{{{*/
// ---------------------------------------------------------------------
/* Configuration only ordering orders by the Depends: line only. It
   orders configuration so that when a package comes to be configured it's
   dependants are configured. 
 
   Loops are ingored. Depends loop entry points are chaotic. */
bool pkgPackageManager::DepConfigure(VisitState &State,DepIterator D)
{
   for (; D.end() == false; D++)
      if (D->Type == pkgDEP_Depends)
      {
	 /* Check if this is an interesting version. All non reverse
	    dependancies are interesting */
	 if (D.Reverse() == true)
	 {
	 #ifdef blah
	    /* Duplication prevention, consider on the rev deps on
	       install state versions */
	    if (Cache[D.ParentPkg()].InstVerIter(Cache) != D.ParentVer())
	       continue;

	    if (VisitNode(State,D.ParentPkg()) == false)
	       return false;
	 #endif
	 }
	 else
            if (VisitProvides(State,D) == false)
	       return false;
      }
   return true;
}
									/*}}}*/
// PM::DepRemove - Removal ordering					/*{{{*/
// ---------------------------------------------------------------------
/* Removal visits all reverse depends. */
bool pkgPackageManager::DepRemove(VisitState &State,DepIterator D)
{
   for (; D.end() == false; D++)
      if (D->Type == pkgDEP_Depends || D->Type == pkgDEP_PreDepends)
	 if (VisitNode(State,D.ParentPkg()) == false)
	    return false;
   
   return true;
}
									/*}}}*/

// PM::DepAlwaysTrue - Returns true if this dep is irrelevent		/*{{{*/
// ---------------------------------------------------------------------
/* The restriction on provides is to eliminate the case when provides
   are transitioning between valid states [ie exim to smail] */
bool pkgPackageManager::DepAlwaysTrue(DepIterator D)
{
   if (D.TargetPkg()->ProvidesList != 0)
      return false;
   
   if ((Cache[D] & pkgDepCache::DepInstall) != 0 &&
       (Cache[D] & pkgDepCache::DepNow) != 0)
      return true;
   return false;
}
									/*}}}*/
// PM::Score - Score the package for sorting				/*{{{*/
// ---------------------------------------------------------------------
/* Higher scores order earlier */
int pkgPackageManager::Score(PkgIterator Pkg)
{
   // Removal is always done first
   if (Cache[Pkg].Delete() == true)
      return 100;
   
   int Score = 0;
   if ((Pkg->Flags & pkgFLAG_Essential) == pkgFLAG_Essential)
      Score += 100;

   // Important Required Standard Optional Extra
   signed short PrioMap[] = {0,5,4,3,1,0};
   if (Cache[Pkg].InstVerIter(Cache)->Priority <= 5)
      Score += PrioMap[Cache[Pkg].InstVerIter(Cache)->Priority];
   return Score;
}
									/*}}}*/
// PM::OrderCompare - Order the installation				/*{{{*/
// ---------------------------------------------------------------------
/* This sort function should be turned to give the lowest number of breaks
   and the highest degree of predictability (same input, same output) */
int pkgPackageManager::OrderCompare(const void *a, const void *b)
{
   PkgIterator A(Me->Cache,*(Package **)a);
   PkgIterator B(Me->Cache,*(Package **)b);
   
   int ScoreA = Me->Score(A);
   int ScoreB = Me->Score(B);
   if (ScoreA > ScoreB)
      return -1;
   
   if (ScoreA < ScoreB)
      return 1;

   return strcmp(A.Name(),B.Name());
}
									/*}}}*/
// PM::DetectInstLoop - Detect a loop in the unpacking odering		/*{{{*/
// ---------------------------------------------------------------------
/* This looks for possible loops in the conflicts+predepends ordering 
   section. These loops and only these loops create packages that are
   not directly installable. 
 
   Only simple cases are detected, such as the perl/perl-base conflicts 
   predepend loop. I -HIGHLY- recommend that this be advoided as much as 
   possible! */
bool pkgPackageManager::DetectInstLoop(Package **List,Package **End)
{
   for (Package **I = List; I != End; I++)
   {
      PkgIterator Pkg(Cache,*I);
      if (Cache[Pkg].InstallVer == 0)
	 continue;
	 
      for (DepIterator D = Cache[Pkg].InstVerIter(Cache).DependsList(); 
	   D.end() == false; D++)
      {
	 if (D->Type != pkgDEP_PreDepends && D->Type != pkgDEP_Conflicts)
	    continue;

	 /* We ignore deps that are ways true, this masks looping but 
	    is probably the best thing */
	 if (DepAlwaysTrue(D) == true)
	    continue;

	 if (Cache[D.TargetPkg()].InstallVer == 0)
	    continue;
	 
	 PkgIterator PkgB = D.TargetPkg();
	 
	 // Look over the targets depends list for a possible loop
	 bool Loop = false;
	 DepIterator D2 = Cache[D.TargetPkg()].InstVerIter(Cache).DependsList();
	 for (; D2.end() == false; D2++)
	 {
	    if (DepAlwaysTrue(D2) == true)
	       continue;
	    if ((D2->Type == pkgDEP_PreDepends ||
		D2->Type == pkgDEP_Conflicts) && D2.TargetPkg() == Pkg)
	    {
	       Loop = true;
	       break;
	    }
	 }
	 
	 // Found a loop
	 if (Loop == true)
	 {
	    /* The only thing we can deal with is a conflicts predepend
	       loop. */
	    if (D2->Type == D->Type)
	       return _error->Error("Found an unpacking loop for %s and %s",Pkg.Name(),PkgB.Name());
	    
	    // Make the conflicts before the predepends.
	    // Locate the placement in the list
	    Package **J = List;
	    for (; J != End; J++)
	       if (*J == (Package *)PkgB)
		  break;

	    // We need to put I after J
	    if (I < J && D->Type == pkgDEP_PreDepends)
	    {
	       Package *Swap = *I;
	       memmove(I,I+1,(J - I)*sizeof(*J));
	       *J = Swap;
	       I = List;
	    }
	    
	    // We need to put J after I
	    if (J < I && D2->Type == pkgDEP_PreDepends)
	    {
	       Package *Swap = *J;
	       memmove(J,J+1,(I - J)*sizeof(*J));
	       *I = Swap;
	       I = List;
	    }
	    cout << "Found unpack loop, " << Pkg.Name() << ' ' << PkgB.Name() << endl;
	 }	 
      }      
   }
   return true;
}
									/*}}}*/
// PM::RunConfigurePhase - Configuration phase of the install		/*{{{*/
// ---------------------------------------------------------------------
/* The configuration phase is when the unpacked .deb files are 
   configured according to the order of their dependancies. This routine
   determines a list of packages that are ready to configure and then 
   orders that list by their dependancies and calls configure on the list. */
bool pkgPackageManager::RunConfigurePhase(Package **CurPacks,
					  Package **End,sbool *Colors)
{
   if (End - CurPacks == 0)
      return true;
   
   unsigned long Size = Cache.HeaderP->PackageCount;
   PkgList InstPacks(Cache);
   Package **IEnd = InstPacks;
   ColorList SColors(Cache);
   memset(SColors,1,sizeof(sbool)*Size);

   /* Add in all unpacked packages that have all dependancies 
      satisfied. */
   for (Package **I = CurPacks; I != End; I++)
   {
      if (Colors[(*I)->ID] != 1)
	 continue;

      PkgIterator Pkg(Cache,*I);

      // Check the dependancies to see if they are all satisfied.
      bool Bad = false;
      for (DepIterator D = Cache[Pkg].InstVerIter(Cache).DependsList(); D.end() == false;)
      {
	 if (D->Type != pkgDEP_Depends)
	 {
	    D++;
	    continue;
	 }
	 
	 // Grok or groups
	 Bad = true;
	 for (bool LastOR = true; D.end() == false && LastOR == true; D++)
	 {
	    LastOR = (D->CompareOp & pkgOP_OR) == pkgOP_OR;
	 
	    if (DepAlwaysTrue(D) == true)
	       Bad = false;
	    
	    PkgIterator DPkg = D.TargetPkg();
	    PrvIterator Prv = DPkg.ProvidesList();
	    
	    /* Loop over all the packages that provide this one and consider
	     then as well */
	    while (Bad == true)
	    {
	       // Target is in its original state.
	       if ((Colors[DPkg->ID] == 0 && 
		    (Cache[D] & pkgDepCache::DepNow) != 0) ||
		   (Colors[DPkg->ID] != 0 && 
		    (Cache[D] & pkgDepCache::DepInstall) != 0))
	       {
		  Bad = false;
		  break;
	       }
	       
	       // Check versioned provides
	       for (; Prv.end() == false; Prv++)
		  if (pkgCheckDep(D.TargetVer(),Prv.ProvideVersion(),D->CompareOp) == true)
		     break;
	       
	       // No more
	       if (Prv.end() == true)
		  break;
	       
	       DPkg = Prv.OwnerPkg();
	       Prv++;
	    }
	 }
	 
	 if (Bad == true)
	    break;
      }
      
      if (Bad == false)
      {
	 SColors[(*I)->ID] = 0;
	 *IEnd++ = *I;
      }	 
   }

   // Final sorted output
   PkgList SortPacks(Cache);
   VisitState Visit;
   Visit.Primary = &DepConfigure;
   Visit.Secondary = 0;
   Visit.RevDepends = 0;
   Visit.Remove = 0;
   Visit.Colors = SColors;
   Visit.Order = SortPacks;
   for (Package **I = InstPacks; I != IEnd; I++)
	 VisitNode(Visit,PkgIterator(Cache,*I));
   InstPacks.Die();
   
   // We have the sorted list now configure it.
   for (Package **I = SortPacks; I != Visit.Order; I++)
   {
      PkgIterator Pkg(Cache,*I);
      
      if (Configure(Pkg) == false)
	 return false;
      
      Colors[Pkg->ID] = 2;
   }
   
   return true;
}
									/*}}}*/
// PM::OrderInstall - Installation ordering routine			/*{{{*/
// ---------------------------------------------------------------------
/* This is a modified version of Manoj's Routine B. It consists of two
   independant stages, installation+removal ordering and configuration
   ordering.
   
   Installation ordering considers each package and its predepends+conflicts
   only. It orders the packages so that when a package is unpacked there
   are no unmet predepends or conflicts.
   
   Removal ordering considers each package and its reverse dependancies
   and reverse predepends. It shceduals and unpack of its reverses before
   it is removed to advoid broken configured dependancies. Removal and
   installation happen in the same pass so packages can be unpacked to 
   clear removal depends requirements.
   
   Configuration ordering considers each package that has been unpacked
   and decides if it is possible to configure it. Because ordered removals
   are involved it is possible a package was unpacked to satisify a remove
   but does not have complete dependancies yet. These packages are ignored
   in the configure ordering phase. Once a list of valid, configurable
   packages has been gathered it then sorts by dependancy only to produce
   an ordered configure list.
   
   By calling the configuration routine whenever an unmet predepends is
   encountered breaks in the installation are created. The configuration
   routine installs all the packages it can, though only the packages
   required to deal with predepends are truely needed. */
bool pkgPackageManager::OrderInstall()
{
   // Colors, 0 is uncoloured, 2 is grey and 1 is black.
   // EGCS 1.01 goes boom if Cache.Header().PackageCount is used.
   unsigned long Size = Cache.HeaderP->PackageCount;
   ColorList Colors(Cache);
   memset(Colors,1,sizeof(sbool)*Size);
   
   // Generate the list of affected packages and sort it
   PkgList InstPacks(Cache);
   Package **End = InstPacks;
   for (PkgIterator I = Cache.PkgBegin(); I.end() == false; I++)
   {
      // Not interesting
      if (Cache[I].Keep() == true)
	 continue;
      
      // Append it to the list
      *End = I;
      End++;
      Colors[I->ID] = 0;
   }
   
   // Sort
   Me = this;
   qsort(InstPacks,End - InstPacks,sizeof(Package *),&OrderCompare);
      
   // Final sorted output
   PkgList SortPacks(Cache);
   VisitState Visit;
   Visit.Primary = &DepUnPackCrit;
   Visit.Secondary = &DepConfigure;
   Visit.RevDepends = &DepUnPackDep;
   Visit.Remove = &DepRemove;
   Visit.Colors = Colors;
   Visit.Order = SortPacks;
      
   // Visit everyone performing configure presorting.
   for (Package **I = InstPacks; I != End; I++)
	 VisitNode(Visit,PkgIterator(Cache,*I));

   // Reset
   memcpy(InstPacks,SortPacks,(End - InstPacks)*sizeof(Package *));
   for (Package **I = InstPacks; I != End; I++)
      Colors[(*I)->ID] = 0;
   cout << "DONE 1" << endl;
   
   Visit.Secondary = 0;
   Visit.Order = SortPacks;
   for (Package **I = InstPacks; I != End; I++)
	 VisitNode(Visit,PkgIterator(Cache,*I));

   memcpy(InstPacks,SortPacks,(End - InstPacks)*sizeof(Package *));
   for (Package **I = InstPacks; I != End; I++)
      Colors[(*I)->ID] = 0;
   cout << "DONE 2" << endl;

   Visit.RevDepends = 0;
   Visit.Order = SortPacks;
   for (Package **I = InstPacks; I != End; I++)
	 VisitNode(Visit,PkgIterator(Cache,*I));
   
   cout << "DONE 3" << endl;
   InstPacks.Die();
   
   /* This colours list now has these meanings,
      0 = Now State, 1 = Unpacked Install, 2 = Configured/Removed */
   memset(Colors,0,sizeof(sbool)*Size);
   
   // obsolete
   if (DetectInstLoop(SortPacks,Visit.Order) == false)
      return false;

   cout << "DONE" << endl;
   
   // For now we take a simple approach and install/configure at once
   for (Package **I = SortPacks; I != Visit.Order; I++)
   {
      PkgIterator Pkg(Cache,*I);
      
      // Sanity check
      if (Cache[Pkg].Keep() == true)
	 return _error->Error("Internal Error, trying to manipulate a kept package");
      
      // Perform a delete or an install
      if (Cache[Pkg].Delete() == true)
      {
	 if (Remove(Pkg) == false)
	    return false;
	 
	 Colors[Pkg->ID] = 2;
      }
      else
      {
	 /* See if this packages install version has any predependancies
	    that are not met by 'now' packages. */
	 for (DepIterator D = Cache[Pkg].InstVerIter(Cache).DependsList(); 
	      D.end() == false; D++)
	 {
	    if (D->Type != pkgDEP_PreDepends)
	       continue;
	    
	    /* Predepends is satisified by the originally installed
	       package */
	    if ((Cache[D] & pkgDepCache::DepNow) != 0 && 
		Colors[D.TargetPkg()->ID] == 0)
	       continue;
	    
	    // Package is already configured.
	    if (Colors[D.TargetPkg()->ID] == 2)
	       continue;
	    
	    if (RunConfigurePhase(SortPacks,I,Colors) == false)
	       return false;
	    
	    // Sanity Check
	    if (Colors[D.TargetPkg()->ID] != 2)
	    {
	       cout << "Error couldn't configure " << D.TargetPkg().Name() << " for predep " << Pkg.Name() << endl;
	       return _error->Error("Internal error, could not configure a predepend");
	    }
	 }
	 
	 if (Install(Pkg,FileNames[Pkg->ID]) == false)
	    return false;
	 
	 Colors[Pkg->ID] = 1;
      }      
   }
   
   // Final run through the configure phase
   if (RunConfigurePhase(SortPacks,Visit.Order,Colors) == false)
      return false;

   // Sanity check
   for (Package **I = SortPacks; I != Visit.Order; I++)
      if (Colors[(*I)->ID] != 2)
	 return _error->Error("Internal error, packages left unconfigured. %s",
			      PkgIterator(Cache,*I).Name());
   
   return true;
}
									/*}}}*/
// PM::DoInstall - Does the installation				/*{{{*/
// ---------------------------------------------------------------------
/* This uses the filenames in FileNames and the information in the
   DepCache to perform the installation of packages.*/
bool pkgPackageManager::DoInstall()
{
   return OrderInstall();
}
									/*}}}*/
