// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: packagemanager.h,v 1.7 1998/03/10 07:57:48 jgg Exp $
/* ######################################################################

   Package Manager - Abstacts the package manager

   Three steps are 
     - Aquiration of archives (stores the list of final file names)
     - Sorting of operations
     - Inokation of package manager
   
   This is the final stage when the package cache entities get converted
   into file names and the state stored in a DepCache is transformed
   into a series of operations.

   In the final scheme of things this may serve as a director class to
   access the actual install methods based on the file type being
   installed.
   
   ##################################################################### */
									/*}}}*/
// Header section: pkglib
#ifndef PKGLIB_PACKAGEMANAGER_H
#define PKGLIB_PACKAGEMANAGER_H

#include <string>
#include <vector>
#include <pkglib/pkgcache.h>

class pkgAquire;
class pkgDepCache;
class pkgSourceList;
class pkgPackageManager
{
   protected:
   string *FileNames;
   pkgDepCache &Cache;
   
   struct VisitState;
   
   // Bring some usefull types into the local scope
   typedef pkgCache::PkgIterator PkgIterator;
   typedef pkgCache::VerIterator VerIterator;
   typedef pkgCache::DepIterator DepIterator;
   typedef pkgCache::PrvIterator PrvIterator;
   typedef pkgCache::Package Package;
   typedef unsigned char sbool;                // small bool
   typedef bool (pkgPackageManager::*DepFunc)(VisitState &State,DepIterator D);

   // Records the state of visit
   struct VisitState
   {
      DepFunc Primary;
      DepFunc Secondary;
      DepFunc RevDepends;
      DepFunc Remove;
      Package **Order;
      sbool *Colors;
   };   

   // For pre sorting
   static pkgPackageManager *Me;
   static int OrderCompare(const void *a, const void *b);
   int Score(PkgIterator Pkg);
      
   // Main visit function
   bool VisitNode(VisitState &State,PkgIterator Pkg);
   bool VisitDeps(VisitState &State,DepFunc F,PkgIterator Pkg);
   bool VisitRDeps(VisitState &State,DepFunc F,PkgIterator Pkg);
   bool VisitRProvides(VisitState &State,DepFunc F,VerIterator Ver);
   bool VisitProvides(VisitState &State,DepIterator Pkg);
   
   // Dependancy checking functions.
   bool DepUnPackCrit(VisitState &State,DepIterator D);
   bool DepUnPackPreD(VisitState &State,DepIterator D);
   bool DepUnPackDep(VisitState &State,DepIterator D);
   bool DepConfigure(VisitState &State,DepIterator D);
   bool DepRemove(VisitState &State,DepIterator D);
   
   bool RunConfigurePhase(Package **SortPacks,Package **End,sbool *Colors);
   bool DetectInstLoop(Package **List,Package **End);
   bool OrderInstall();
   
   // Analysis helpers
   bool DepAlwaysTrue(DepIterator D);
   bool UnWind();
   
   // The Actuall installation implementation
   virtual bool Install(PkgIterator Pkg,string File) = 0;
   virtual bool Configure(PkgIterator Pkg) = 0;
   virtual bool Remove(PkgIterator Pkg) = 0;
   
   public:
      
   // The three stages
   bool GetArchives(pkgSourceList &List,pkgAquire &Engine);
   bool DoInstall();
      
   pkgPackageManager(pkgDepCache &Cache);
   virtual ~pkgPackageManager();
};

#endif
