// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: sourcelist.cc,v 1.5 1998/03/07 09:27:40 jgg Exp $
/* ######################################################################

   List of Sources
   
   ##################################################################### */
									/*}}}*/
// Include Files							/*{{{*/
#include <pkglib/sourcelist.h>
#include <pkglib/aquire.h>
#include <pkglib/error.h>
#include <pkglib/pkgcachemerge.h>
#include <pkglib/pkglist.h>
#include <strutl.h>
#include <fileutl.h>
#include <options.h>

#include <fstream.h>
#include <stdio.h>
#include <unistd.h>
									/*}}}*/

// SourceList::pkgSourceList - Constructors				/*{{{*/
// ---------------------------------------------------------------------
/* */
pkgSourceList::pkgSourceList()
{
}

pkgSourceList::pkgSourceList(string File)
{
   Read(File);
}
									/*}}}*/
// SourceList::ReadMainList - Read the main source list from etc	/*{{{*/
// ---------------------------------------------------------------------
/* */
bool pkgSourceList::ReadMainList()
{
   return Read(PKG_DEB_CF_SOURCELIST);
}
									/*}}}*/
// SourceList::Read - Parse the sourcelist file				/*{{{*/
// ---------------------------------------------------------------------
/* */
bool pkgSourceList::Read(string File)
{
   // Open the stream for reading
   ifstream F(File.c_str(),ios::in | ios::nocreate);
   if (!F != 0)
      return _error->Errno("ifstream::ifstream","Opening %s",File.c_str());
   
   List.erase(List.begin(),List.end());
   char Buffer[300];

   int CurLine = 0;
   while (F.eof() == false)
   {
      F.getline(Buffer,sizeof(Buffer));
      CurLine++;
      
      // Comment or blank
      if (Buffer[0] == '#' || Buffer[0] == 0)
	 continue;
      
      // Grok it
      string Type;
      string URN;
      Item Itm;
      char *C = Buffer;
      if (ParseQuoteWord(C,Type) == false ||
	  ParseQuoteWord(C,URN) == false ||
	  ParseQuoteWord(C,Itm.Dist) == false ||
	  Itm.SetType(Type) == false ||
	  Itm.SetURN(URN) == false)
	 return _error->Error("Malformed line %u in source list %s",CurLine,File.c_str());

      // Check for an absolute dists specification.
      if (Itm.Dist.empty() == false && Itm.Dist[Itm.Dist.size() - 1] == '/')
      {
	 if (ParseQuoteWord(C,Itm.Section) == true)
	    return _error->Error("Malformed line %u in source list %s",CurLine,File.c_str());
	 List.push_back(Itm);
	 continue;
      }

      // Grab the rest of the dists
      if (ParseQuoteWord(C,Itm.Section) == false)
	    return _error->Error("Malformed line %u in source list %s",CurLine,File.c_str());
      
      do
      {
	 List.push_back(Itm);
      }
      while (ParseQuoteWord(C,Itm.Section) == true);
   }
   return true;
}
									/*}}}*/
// SourceList::SanitizeURN - Hash the urn				/*{{{*/
// ---------------------------------------------------------------------
/* This converts a URN into a safe filename. It quotes all unsafe characters
   and converts / to _ and removes the scheme identifier. */
string pkgSourceList::SanitizeURN(string URN)
{
   string::const_iterator I = URN.begin() + URN.find(':') + 1;
   for (; I < URN.end() && *I == '/'; I++);
   
   URN = QuoteString(string(I,URN.end() - I),"~_=!@#$%^&*");
   string::iterator J = URN.begin();
   for (; J != URN.end(); J++)
      if (*J == '/') 
	 *J = '_';
   return URN;
}
									/*}}}*/
// SourceList::MatchPkgFile - Find the package file that has the ver	/*{{{*/
// ---------------------------------------------------------------------
/* This will return List.end() if it could not find the matching 
   file */
pkgSourceList::const_iterator pkgSourceList::MatchPkgFile(pkgCache::VerIterator Ver)
{
   string Base = PKG_DEB_ST_LIST;
   for (const_iterator I = List.begin(); I != List.end(); I++)
   {
      string URN = I->PackagesURN();
      switch (I->Type)
      {
	 case Item::Deb:
	 if (Base + SanitizeURN(URN) == Ver.File().FileName())
	    return I;
	 break;
      };      
   }
   return List.end();
}
									/*}}}*/
// SourceList::Item << - Writes the item to a stream			/*{{{*/
// ---------------------------------------------------------------------
/* This is not suitable for rebuilding the sourcelist file but it good for
   debugging. */
ostream &operator <<(ostream &O,pkgSourceList::Item &Itm)
{
   O << Itm.Type << ' ' << Itm.URN << ' ' << Itm.Dist << ' ' << Itm.Section;
   return O;
}
									/*}}}*/
// SourceList::Item::SetType - Sets the distribution type		/*{{{*/
// ---------------------------------------------------------------------
/* */
bool pkgSourceList::Item::SetType(string S)
{
   if (S == "deb")
   {
      Type = Deb;
      return true;
   }

   return true;
}
									/*}}}*/
// SourceList::Item::SetURN - Set the URN				/*{{{*/
// ---------------------------------------------------------------------
/* For simplicity we strip the scheme off the urn */
bool pkgSourceList::Item::SetURN(string S)
{
   if (S.empty() == true)
      return false;
   
   if (S.find(':') == string::npos)
      return false;
   
   // Make sure that the URN is / postfixed
   URN = S;
   if (URN[URN.size() - 1] != '/')
      URN += '/';
   
   return true;
}
									/*}}}*/
// SourceList::Item::PackagesURN - Returns a URN to the packages file	/*{{{*/
// ---------------------------------------------------------------------
/* */
string pkgSourceList::Item::PackagesURN() const
{
   string Res;
   switch (Type)
   {
      case Deb:
      if (Dist[Dist.size() - 1] == '/')
	 Res = URN + Dist;
      else
	 Res = URN + "dists/" + Dist + '/' + Section + 
	 "/binary-" + PKG_DEB_ARCH + '/';
      
      Res += "Packages";
      break;
   };
   return Res;
}
									/*}}}*/
// SourceList::Item::ArchiveURN - Returns a URN to the given archive	/*{{{*/
// ---------------------------------------------------------------------
/* */
string pkgSourceList::Item::ArchiveURN(string File) const
{
   string Res;
   switch (Type)
   {
      case Deb:
      Res = URN + File;
      break;
   };
   return Res;
}
									/*}}}*/

// UpdateMeta - Update the meta information				/*{{{*/
// ---------------------------------------------------------------------
/* The meta information is package files, revision information and mirror
   lists. */
bool pkgUpdateMeta(pkgSourceList &List,pkgAquire &Engine)
{
   if (Engine.OutputDir(PKG_DEB_ST_LIST) == false)
      return false;
   
   for (pkgSourceList::const_iterator I = List.begin(); I != List.end(); I++)
   {
      string URN = I->PackagesURN();
      switch (I->Type)
      {
	 case pkgSourceList::Item::Deb:
	 if (Engine.Get(URN + ".gz",List.SanitizeURN(URN)) == false)
	    return false;
	 break;
      };      
   }
   
   return true;
}
									/*}}}*/
// MakeSrcCache - Generate a cache file of all the package files	/*{{{*/
// ---------------------------------------------------------------------
/* This goes over the source list and builds a cache of all the package
   files. */
bool pkgMakeSrcCache(pkgSourceList &List)
{
   // First we date check the cache
   bool Bad = false;
   while (Bad == false)
   {
      if (FileExists(PKG_DEB_CA_SRCCACHE) == false)
	  break;
	  
      pkgCache Cache(PKG_DEB_CA_SRCCACHE,true,true);
      if (_error->PendingError() == true)
      {
	 _error->Discard();
	 break;
      }
      
      // They are certianly out of sync
      if (Cache.Head().PackageFileCount != List.size())
	  break;
      
      for (pkgCache::PkgFileIterator F(Cache); F.end() == false; F++)
      {
	 // Search for a match in the source list
	 Bad = true;
	 for (pkgSourceList::const_iterator I = List.begin(); 
	      I != List.end(); I++)
	 {
	    string File = string(PKG_DEB_ST_LIST) + 
	       List.SanitizeURN(I->PackagesURN());
	    if (F.FileName() == File)
	    {
	       Bad = false;
	       break;
	    }
	 }
	 
	 // Check if the file matches what was cached
	 Bad |= !F.IsOk();
	 if (Bad == true)
	    break;
      }      

      if (Bad == false)
	 return true;
   }
   
   unlink(PKG_DEB_CA_SRCCACHE);
   pkgCache::MergeState Merge(PKG_DEB_CA_SRCCACHE);
   if (_error->PendingError() == true)
            return false;
   
   for (pkgSourceList::const_iterator I = List.begin(); I != List.end(); I++)
   {
      string File = string(PKG_DEB_ST_LIST) + List.SanitizeURN(I->PackagesURN());
      if (Merge.MergePackageFile(File,"??","??") == false)
	 return false;
   }
          
   return true;
}
									/*}}}*/
// MakeStatusCache - Generates a cache that includes the status files	/*{{{*/
// ---------------------------------------------------------------------
/* This copies the package source cache and then merges the status and 
   xstatus files into it. */
bool pkgMakeStatusCache()
{
   // Quickly check if the existing package cache is ok
   bool Bad = false;
   while (Bad == false)
   {
      if (FileExists(PKG_DEB_CA_PKGCACHE) == false)
	  break;
      
      pkgCache Cache(PKG_DEB_CA_PKGCACHE,true,true);
      if (_error->PendingError() == true)
      {
	 _error->Discard();
	 break;
      }
      
      for (pkgCache::PkgFileIterator F(Cache); F.end() == false; F++)
      {
	 if (F.IsOk() == false)
	 {
	    Bad = true;
	    break;
	 }
      }
      
      if (Bad == false)
	 return true;
   }   

   // Check the integrity of the source cache.
   {
      pkgCache Cache(PKG_DEB_CA_SRCCACHE,true,true);
      if (_error->PendingError() == true)
	 return false;
   }
   
   // Sub scope so that merge destructs before we rename the file...
   string Cache = PKG_DEB_CA_PKGCACHE ".new";
   {
      if (CopyFile(PKG_DEB_CA_SRCCACHE,Cache) == false)
	 return false;

      pkgCache::MergeState Merge(Cache);
      if (_error->PendingError() == true)
	 return false;
      
      // Merge in the extra status file
      if (FileExists(PKG_DEB_ST_XSTATUS) == true)
	 if (Merge.MergePackageFile(PKG_DEB_ST_XSTATUS,"status","0") == false)
	    return false;
      
      // Merge in the status file
      if (Merge.MergePackageFile("/var/lib/dpkg/status","status","0") == false)
	 return false;									       
   }
   
   if (rename(Cache.c_str(),PKG_DEB_CA_PKGCACHE) != 0)
      return false;
   
   return true;
}
									/*}}}*/
