// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: sourcelist.h,v 1.6 1998/03/05 06:54:19 jgg Exp $
/* ######################################################################

   SourceList - Manage a list of sources
   
   The Source List class provides access to a list of sources. It 
   can read them from a file and generate a list of all the permutations.
   
   ##################################################################### */
									/*}}}*/
// Header section: pkglib
#ifndef PKGLIB_SOURCELIST_H
#define PKGLIB_SOURCELIST_H

#include <string>
#include <vector>
#include <iostream.h>
#include <pkglib/pkgcache.h>

class pkgAquire;
class pkgSourceList
{
   public:
   
   /* Each item in the source list, each line can have more than one
      item */
   struct Item
   {
      enum {Deb} Type;

      string URN;
      string Dist;
      string Section;
      
      bool SetType(string S);
      bool SetURN(string S);
      string PackagesURN() const;
      string ArchiveURN(string File) const;
   };
   typedef vector<Item>::const_iterator const_iterator;
   
   protected:
   
   vector<Item> List;
   
   public:

   bool ReadMainList();
   bool Read(string File);
   string SanitizeURN(string URN);
   const_iterator MatchPkgFile(pkgCache::VerIterator Ver);
   
   // List accessors
   inline const_iterator begin() const {return List.begin();};
   inline const_iterator end() const {return List.end();};
   inline unsigned int size() const {return List.size();};
      
   pkgSourceList();
   pkgSourceList(string File);   
};

bool pkgUpdateMeta(pkgSourceList &List,pkgAquire &Engine);
bool pkgMakeSrcCache(pkgSourceList &List);
bool pkgMakeStatusCache();

ostream &operator <<(ostream &O,pkgSourceList::Item &Itm);

#endif
