// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: versiontest.cc,v 1.3 1997/10/10 22:04:01 jgg Exp $
/* ######################################################################

   Version Test - Simple program to run through a file and comare versions.
   
   Each version is compared and the result is checked against an expected
   result in the file. The format of the file is
       a b Res
   Where Res is -1, 1, 0. dpkg -D=1 --compare-versions a "<" b can be
   used to determine what Res should be. # at the start of the line
   is a comment and blank lines are skipped
   
   ##################################################################### */
									/*}}}*/
#include <system.h>
#include <pkglib/error.h>
#include <pkglib/version.h>
#include <iostream.h>
#include <fstream.h>

bool RunTest(const char *File)
{
   ifstream F(File,ios::in | ios::nocreate);
   if (!F != 0)
      return false;

   char Buffer[300];
   int CurLine = 0;
   
   while (1)
   {
      F.getline(Buffer,sizeof(Buffer));
      CurLine++;
      if (F.eof() != 0)
	 return true;
      if (!F != 0)
	 return _error->Error("Line %u in %s is too long",CurLine,File);

      // Comment
      if (Buffer[0] == '#' || Buffer[0] == 0)
	 continue;
      
      // First version
      char *I;
      char *Start = Buffer;
      for (I = Buffer; *I != 0 && *I != ' '; I++);
      string A(Start, I - Start);

      if (*I == 0)
	 return _error->Error("Invalid line %u",CurLine);
      
      // Second version
      I++;
      Start = I;
      for (I = Start; *I != 0 && *I != ' '; I++);
      string B(Start,I - Start);
      
      if (*I == 0 || I[1] == 0)
	 return _error->Error("Invalid line %u",CurLine);
      
      // Result
      I++;
      int Expected = atoi(I);
      int Res = pkgVersionCompare(A.c_str(),B.c_str());
      cout << "'" << A << "' ? '" << B << "' = " << Res << " (= " << Expected << ')' << endl;
      if (Res != Expected)
	 _error->Error("Comparison failed on line %u. '%s' ? '%s' %i != %i",CurLine,A.c_str(),B.c_str(),Res,Expected);

      // Check the reverse as well
      Expected = -1*Expected;
      Res = pkgVersionCompare(B.c_str(),A.c_str());
      cout << "'" << B << "' ? '" << A << "' = " << Res << " (= " << Expected << ')' << endl;
      if (Res != Expected)
	 _error->Error("Comparison failed on line %u. '%s' ? '%s' %i != %i",CurLine,A.c_str(),B.c_str(),Res,Expected);
   }
}

int main(int argc, char *argv[])
{
   if (argc <= 1)
   {
      cerr << "You must specify a test file" << endl;
      return 0;
   }
   
   RunTest(argv[1]);

   // Print any errors or warnings found
   if (_error->empty() == false)
   {
      string Err;
      while (_error->empty() == false)
      {
	 
	 bool Type = _error->PopMessage(Err);
	 if (Type == true)
	    cout << "E: " << Err << endl;
	 else
	    cout << "W: " << Err << endl;
      }
      
      return 0;
   }
}
